# -*-coding:utf-8 -*-
"""
:创建时间: 2022/8/29 15:30
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:Github: https://github.com/cpcgskill
:QQ: 2921251087
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127
:爱发电: https://afdian.net/@Phantom_of_the_Cang

"""
from __future__ import unicode_literals, print_function, division
import os
import sys
import subprocess

if False:
    from typing import List, Tuple, Dict, AnyStr, Any, Callable, Union

__all__ = ['compile_item', 'call_command']


def compile_item(item, data):
    new_data = dict()
    for k, v in os.environ.items():
        new_data[k] = v
    for k, v in data.items():
        new_data[k] = v
    return item.format(**new_data)


def call_command(argv, data=None):
    """
    :type argv: List[AnyStr]
    :type data: Dict[AnyStr, AnyStr or int or float] or None
    """
    if data is None:
        data = dict()
    argv = [compile_item(i, data) for i in argv]
    command = argv[0]
    argv = argv[1:]

    subprocess.check_call([command] + argv)


if __name__ == '__main__':
    do_command(['echo', '{PATH}'])
    do_command(['echo', '{PATH}'], {'PATH': 123})



# -*-coding:utf-8 -*-
u"""
:创建时间: 2022/3/8 12:02
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:QQ: 2921251087
:爱发电: https://afdian.net/@Phantom_of_the_Cang
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127

"""
from __future__ import unicode_literals, print_function, division

if False:
    from typing import *

import datetime
import json
import locale
import os.path
import sys
import uuid
from collections import OrderedDict

import argparse

from pyeal.res import LocalRes, DirectoryRes, MergeRes
from pyeal.seal import seal
from pyeal.exc import *
from pyeal._command import call_command

PATH = os.path.dirname(os.path.abspath(__file__))


class Config(object):
    def __init__(self, root, config_file='pyeal.json'):
        root = os.path.abspath(root)
        self.root_path = os.path.abspath(root)
        self.config_file = config_file
        self.src_path = os.sep.join((self.root_path, 'src'))
        self.lib_path = os.sep.join((self.root_path, 'lib'))
        self.build_path = os.sep.join((self.root_path, 'build'))
        self.middle_path = os.sep.join((root, 'build', 'middle'))
        self.out_path = os.sep.join((root, 'build', 'out'))

        self.root = LocalRes(self.root_path)
        self.src = LocalRes(self.src_path)
        self.lib = LocalRes(self.lib_path)
        self.build = LocalRes(self.build_path)
        self.middle = LocalRes(self.middle_path)
        self.out = LocalRes(self.out_path)

    def get_root_res(self):
        return LocalRes(self.root_path)

    def get_source_res(self):
        outer_lib_path_list = self.get('outer_lib', [])
        lib_paths = self.get('lib', ['lib'] + outer_lib_path_list)
        src_paths = self.get('src', ['src'] + lib_paths)
        src_res_list = [LocalRes(i) for i in src_paths]
        return MergeRes(*src_res_list)

    def get_config(self):
        return json.loads(self.root.read_string(self.config_file))

    @property
    def config_data(self):
        return self.get_config()

    @property
    def type(self):
        return self.config_data["type"]

    @property
    def name(self):
        return self.config_data["name"]

    def get_script(self):
        script = self.config_data.get("exec_script", None)
        if script is None:
            index_module = self.config_data.get("index_module", "index.py")
            script = self.root.read(index_module)
        return script

    def get_icon(self):
        return self.root.read('icon.ico')

    def get(self, key, default=None):
        return self.config_data.get(key, default)

    def ass(self, key):
        c = self.get(key, None)
        if c is None:
            raise ConfigException("需要名为<{}>的配置项".format(key))
        return c


def target_is_exec(config):
    """
    :type config: Config
    """
    seal(
        config.get_source_res(),
        config.out,
        config.name,
        config.get_script(),
        config.get('imp_name', config.name),
    )


def target_is_maya_plugin(config):
    """
    :type config: Config
    """
    m0 = DirectoryRes(config.middle, "m0")

    seal(
        config.get_source_res(),
        m0,
        config.name,
        config.get_script(),
        config.get('imp_name', config.name)
    )
    plugin_path = "dist/plugin"
    for f in m0.files():
        config.out.write('/'.join((plugin_path, f)), m0.read(f))
    with open(os.sep.join((PATH, 'assets', "mel_template_lib.mel")), "rb") as f:
        mel_template_lib_code = f.read().decode("utf-8")
    mel_template = r'''
startInstall(
"exec(compile(open(plugin_path+" + <<exec_file_name>> + ",'rb').read(), plugin_path+" + <<exec_file_name>> + ", 'exec'), globals(), locals())", 
<<ann>>, 
"dist/log.ico", 
<<plugin_path>>
);
'''

    mel_template = mel_template.replace(
        '<<exec_file_name>>',
        '"\'{}.py\'"'.format(config.name)
    )
    mel_template = mel_template.replace(
        '<<ann>>',
        '"{}"'.format(config.get('annotation', '这个插件作者很懒没有写注释哦~'))
    )
    mel_template = mel_template.replace(
        '<<plugin_path>>',
        '"{}/"'.format(plugin_path)
    )
    # 以系统编码写入安装文件
    config.out.write('install.mel', (mel_template_lib_code + mel_template).encode(locale.getpreferredencoding()))
    config.out.write('/'.join(('dist', 'log.ico')), config.get_icon())


def target_is_template(config):
    """
    :type config: Config
    """
    template = DirectoryRes(config.root, 'template')
    template_output = DirectoryRes(config.out, config.ass('template-output'))

    for f in template.files():
        config.out.write(f, template.read(f))

    seal(
        config.get_source_res(),
        template_output,
        config.name,
        config.get_script(),
        config.get('imp_name', config.name),
    )


target_types = {
    'package': target_is_exec,
    'maya-plugin': target_is_maya_plugin,
    'template': target_is_template,
}  # type: Dict[AnyStr, Callable[[Config], None]]


def _call_command_list(command_list, build_data):
    for i in command_list:
        if isinstance(i, list):
            call_command(i, build_data)
        elif isinstance(i, dict):
            try:
                is_do_command = eval(i.get('if_expression', 'True'), dict(), build_data)
            except:
                is_do_command = False
            if is_do_command:
                command = i.get('true_command', None)
                if command:
                    call_command(command, build_data)
            else:
                command = i.get('false_command', None)
                if command:
                    call_command(command, build_data)
        else:
            raise SealException("未知的命令格式")


config_templates = {
    "package": OrderedDict([
        ("type", "package"),
        ("exec_script", ""),
    ]),
    "maya-plugin": OrderedDict([
        ("type", "maya-plugin"),
        ("exec_script", ""),
    ]),
    "template": OrderedDict([
        ("type", "template"),
        ("exec_script", ""),
        ("template-output", "src"),
    ]),
}


class SubCommand(object):
    name = None
    help = ''

    def __init__(self, parser):
        # type: (argparse.ArgumentParser) -> None
        self.parser = parser
        self.parser.set_defaults(func=self.process)
        self.init_parser()

    def init_parser(self):
        raise NotImplementedError()

    def process(self, args):
        raise NotImplementedError()


class BuildCommand(SubCommand):
    name = 'build'
    help = 'build project'

    def init_parser(self):
        # 获得所有位置参数，传递给build data。
        self.parser.add_argument('args', nargs='*', help='args')
        # config
        self.parser.add_argument('-cf', '--config_file', type=str, default='pyeal.json', help='config file')

    def process(self, args):
        config = Config(os.path.abspath("."), args.config_file)
        config.build.clean()
        command_list_at_start = config.get('command_list_at_start', [])
        command_list_at_end = config.get('command_list_at_end', [])
        build_data = {
            'build_uuid': uuid.uuid4().hex,
            'build_time': datetime.datetime.now(),
            'src_path': config.src_path,
            'lib_path': config.lib_path,
            'build_path': config.build_path,
            'middle_path': config.middle_path,
            'out_path': config.out_path,
            'args': args.args,
            'name': config.name,
        }

        _call_command_list(command_list_at_start, build_data)

        target_type_func = target_types.get(config.type)

        if target_type_func is None:
            raise SealException("未知编译类型")

        target_type_func(config)

        _call_command_list(command_list_at_end, build_data)


class InitCommand(SubCommand):
    name = 'init'
    help = 'init project, you can use -t to choose a template'

    def init_parser(self):
        self.parser.add_argument('-t', '--template',
                                 choices=config_templates.keys(),
                                 type=str,
                                 default='package',
                                 help='choose a template'
                                 )
        self.parser.add_argument('-n', '--name', type=str, default='your_name', help='package name')
        self.parser.add_argument('-cf', '--config_file', type=str, default='pyeal.json',
                                 help='config file')

    def process(self, args):
        root_path = os.path.abspath(".")
        root = LocalRes(root_path)

        template = config_templates[args.template]
        template['name'] = args.name

        root.write_string(
            args.config_file,
            json.dumps(template, indent=2, ensure_ascii=False)
        )

        src = os.sep.join((root_path, "src"))
        if not os.path.isdir(src):
            os.makedirs(src)
        build = os.sep.join((root_path, "build"))
        if not os.path.isdir(build):
            os.makedirs(build)

        if args.template == 'maya-plugin':
            with open(os.sep.join((PATH, 'assets', "icon.ico")), "rb") as f:
                root.write("icon.ico", f.read())


class CleanCommand(SubCommand):
    name = 'clean'
    help = 'clean project'

    def init_parser(self):
        self.parser.add_argument('-cf', '--config_file', type=str, default='pyeal.json', help='config file')

    def process(self, args):
        config = Config(os.path.abspath("."), args.config_file)
        config.build.clean()


commands = [
    BuildCommand,
    InitCommand,
    CleanCommand,
]  # type: List[SubCommand]


def main(argv):
    root = os.path.abspath(".")

    parser = argparse.ArgumentParser(
        description='Python 打包编译工具',
        prog='pyeal',
    )
    subparsers = parser.add_subparsers(
        title='subcommands',
        description='valid subcommands',
        help='sub-command help',
    )
    for sub_command in commands:
        sub_command(subparsers.add_parser(sub_command.name, help=sub_command.help))

    args = parser.parse_args(argv)
    args.func(args)



# -*-coding:utf-8 -*-
"""
:创建时间: 2022/9/13 13:32
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:Github: https://github.com/cpcgskill
:QQ: 2921251087
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127
:爱发电: https://afdian.net/@Phantom_of_the_Cang

"""
from __future__ import unicode_literals, print_function, division
import uuid
import ast

from pyeal.codekit.core import Code
from pyeal.res import BaseRes
from pyeal.module_data import ModuleData

if False:
    from typing import List, Tuple, Dict, AnyStr, Any


class Sealer(object):
    """打包编译器"""

    def __init__(self, source, target, name, code, imp_name=None):
        """
        :type source: BaseRes
        :type target: BaseRes
        :type name: unicode
        """
        self.source = source
        self.target = target
        self.name = name
        self.code = code
        if imp_name is None:
            self.imp_name = name
        else:
            self.imp_name = imp_name
        self.module_data = ModuleData(self.source)
        self.uid = uuid.uuid4().hex[0:4]

    def seal_name(self):
        return "{}_{}".format(self.name, self.uid)

    def target_name(self, p):
        return "{}_{}".format(self.seal_name(), p)

    def target_path(self, p):
        if len(p.split('/')) == 1 and \
                p.split('.')[-1] != 'py':
            return p
        else:
            return "{}_{}".format(self.seal_name(), p)

    def target_dir_path(self, p):
        return "{}_{}".format(self.seal_name(), p)

    def compile_import_node(self, n, m):
        alias_list = []
        for i in n.names:
            if m is None:
                r_find_name = None
            else:
                r_find_name = self.module_data.relative_find_module(i.name, m)
            name = self.module_data.find_module(i.name)
            if r_find_name is None and name is not None:
                if i.asname is None:
                    alias_list.append(ast.alias(asname=i.name.split(".")[0],
                                                name=self.target_name(i.name.split('.')[0])))
                    alias_list.append(ast.alias(asname="_",
                                                name=self.target_name(name)))
                else:
                    target_asname = i.asname
                    alias_list.append(ast.alias(asname="_",
                                                name=self.target_name(i.name.split('.')[0])))
                    alias_list.append(ast.alias(asname=target_asname,
                                                name=self.target_name(name)))

            else:
                alias_list.append(i)
        return [ast.Import(names=[i]) for i in alias_list]

    def compile_import_from_node(self, n, m):
        if n.level == 0:
            nodes = []
            if m is None:
                r_find_name = None
            else:
                r_find_name = self.module_data.relative_find_module(n.module, m)
            name = self.module_data.find_module(n.module)
            if r_find_name is None and name is not None:
                nodes.append(ast.ImportFrom(level=0,
                                            module=self.target_name(name),
                                            names=n.names))
            else:
                nodes.append(n)
            return nodes
        else:
            return [n]

    def delete_future(self, n):
        if isinstance(n, ast.ImportFrom):
            if n.level == 0:
                if n.module == "__future__":
                    return True
        return False

    def compile_module(self, code, m=None):
        """
        :type code: bytes
        :type m: AnyStr or None
        :return:
        """
        code_editor = Code(code)

        code_editor.replace_node(
            check_key=lambda n: isinstance(n, ast.Import),
            key=lambda n: self.compile_import_node(n, m)
        ).replace_node(
            check_key=lambda n: isinstance(n, ast.ImportFrom),
            key=lambda n: self.compile_import_from_node(n, m)
        )

        deleted_future = code_editor.delete_node(key=self.delete_future)

        code_editor.insert_to_head(deleted_future)

        code = code_editor.unparse()

        return code.encode("utf-8")

    def build(self):
        for d in self.source.dirs():
            if d.split('/')[-1] != '__pycache__':
                self.target.make_dir(self.target_dir_path(d))
        for f in self.source.files():
            if f.split('.')[-1] == 'pyc':
                test_f = '.'.join(f.split('.')[:-1] + ['py'])
                if not self.source.is_file(test_f):
                    self.target.write(self.target_path(f), self.source.read(f))
            else:
                self.target.write(self.target_path(f), self.source.read(f))
        for m, f in self.module_data.module_name_and_paths():
            code = self.source.read(f)
            code = self.compile_module(code, m)
            self.target.write(self.target_path(f), code)
        code = self.compile_module(self.code, None)
        self.target.write(self.imp_name + ".py", code)


def seal(source, target, name, code, imp_name=None):
    """
    :type source: BaseRes
    :type target: BaseRes
    :type name: AnyStr
    :type code: AnyStr
    :type imp_name: AnyStr or None
    """
    return Sealer(source, target, name, code, imp_name).build()



# -*-coding:utf-8 -*-
u"""
:创建时间: 2021/12/22 7:38
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:QQ: 2921251087
:爱发电: https://afdian.net/@Phantom_of_the_Cang
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127

"""
from __future__ import unicode_literals, print_function

import os
import sys
from abc import abstractmethod

from pyeal.exc import *

if False:
    from typing import List, Tuple, Dict, AnyStr, Any


str_t = (type(u''), str)
bytes_t = (type(b''), bytes)

class BaseRes(object):
    @abstractmethod
    def customize_read(self, path):
        """
        :type path: AnyStr
        :rtype: bytes
        """

    @abstractmethod
    def customize_write(self, path, data):
        """
        :type path: AnyStr
        :type data: bytes
        """

    @abstractmethod
    def customize_remove(self, path):
        """
        :type path: AnyStr
        """

    @abstractmethod
    def customize_make_dir(self, path):
        """
        :type path: AnyStr
        """

    @abstractmethod
    def customize_remove_dir(self, path):
        """
        :type path: AnyStr
        """

    @abstractmethod
    def customize_sep(self):
        """
        :rtype: AnyStr
        """

    @abstractmethod
    def customize_is_dir(self, path):
        """
        :type path: AnyStr
        :rtype: bool
        """

    @abstractmethod
    def customize_is_file(self, path):
        """
        :type path: AnyStr
        :rtype: bool
        """

    @abstractmethod
    def customize_walk(self):
        """

        :rtype: List[Tuple[AnyStr, List[AnyStr], List[AnyStr]]]
        """

    def to_customize_sep(self, path):
        """
        :type path: AnyStr
        :rtype: AnyStr
        """
        return self.customize_sep().join((i for i in path.split('/') if i != ''))

    def to_std_sep(self, path):
        """
        :type path: AnyStr
        :rtype: AnyStr
        """
        return '/'.join((i for i in path.split(self.customize_sep()) if i != ''))

    @staticmethod
    def join_path(*path):
        return '/'.join(path)

    def read(self, path):
        """
        :type path: AnyStr
        :rtype: bytes
        """
        return self.customize_read(self.to_customize_sep(path))

    def write(self, path, data):
        """
        :type path: AnyStr
        :type data: bytes
        """
        self.customize_write(self.to_customize_sep(path), data)
        return self

    def read_string(self, path):
        """
        :type path: AnyStr
        :rtype: AnyStr
        """
        return self.read(path).decode('utf-8')

    def write_string(self, path, data):
        """
        :type path: AnyStr
        :type data: AnyStr
        """
        if isinstance(data, str_t):
            data = data.encode('utf-8')
        elif not isinstance(data, bytes_t):
            raise TypeError('data must be str or bytes')
        self.write(path, data)
        return self

    def remove(self, path):
        self.customize_remove(self.to_customize_sep(path))
        return self

    def make_dir(self, path):
        path = self.to_customize_sep(path)
        if not self.customize_is_dir(path):
            self.customize_make_dir(path)
        return self

    def remove_dir(self, path):
        path = self.to_customize_sep(path)
        if self.customize_is_dir(path):
            self.customize_remove_dir(path)
        return self

    def is_dir(self, path):
        path = self.to_customize_sep(path)
        return self.customize_is_dir(path)

    def is_file(self, path):
        path = self.to_customize_sep(path)
        return self.customize_is_file(path)

    def walk(self, prefix=''):
        """
        :type prefix: AnyStr
        :rtype: List[Tuple[AnyStr, List[AnyStr], List[AnyStr]]]
        """
        prefix = self.to_std_sep(prefix)
        data = [(self.to_std_sep(root), dirs, files) for root, dirs, files in self.customize_walk()]
        # 如果前缀不为空，则过滤所有不和前缀匹配的root
        if prefix != '':
            prefix = prefix.split('/')

            def path_prefix_match(matched_path):
                matched_path = matched_path.split('/')
                if len(prefix) > len(matched_path):
                    return True
                matched_path = matched_path[:len(prefix)]
                return all([i[0] == i[1] for i in zip(prefix, matched_path)])

            return [(root, dirs, files) for root, dirs, files in data if path_prefix_match(root)]
        return data

    def __files(self, prefix=''):
        for root, dirs, files in self.walk(prefix):
            for f in files:
                if root != '':
                    f = '/'.join((root, f))
                yield f

    def files(self, prefix=''):
        """
        :type prefix: AnyStr
        :rtype: List[AnyStr]
        """
        return list(self.__files(prefix))

    def __dirs(self, prefix=''):
        for root, dirs, files in self.walk(prefix):
            for d in dirs:
                if root != '':
                    d = '/'.join((root, d))
                yield d

    def dirs(self, prefix=''):
        """
        :type prefix: AnyStr
        :rtype: List[AnyStr]
        """
        return list(self.__dirs(prefix))

    def clean(self, path=''):
        """
        :type path: AnyStr
        """
        for f in self.files(path):
            # self.customize_remove(f)
            self.remove(f)
        for d in reversed(self.dirs(path)):
            # self.customize_remove_dir(d)
            self.remove_dir(d)
        return self


class LocalRes(BaseRes):
    def __init__(self, root):
        """

        :type root: AnyStr
        """
        self.root = os.path.abspath(root)
        if sys.platform == 'win32':
            # if len(self.root) > 256:
            self.root = '\\\\?\\' + self.root

    def __str__(self):
        return str(self.__unicode__())

    def __unicode__(self):
        return "{}(root={})".format(self.__class__.__name__, repr(self.root))

    def __post_process_path(self, path):
        return os.path.abspath(os.sep.join((self.root, path)))

    def customize_read(self, path):
        path = self.__post_process_path(path)
        # if len(path) > 256:
        #     path = '\\\\?\\' + path
        with open(path, "rb") as f:
            return f.read()

    def customize_write(self, path, data):
        # if path.find('elementwise_logical_ops_test.test_is_member_of.zip') != -1:
        #     pass
        path = self.__post_process_path(path)
        dir_, name = os.path.split(path)
        if not os.path.isdir(dir_):
            os.makedirs(dir_)
        # if sys.platform == 'win32':
        #     try:
        #         with open(path, "wb") as f:
        #             return f.write(data)
        #     except IOError as ex:
        #         if len(path) > 767:
        #             raise IOError('Accidents caused by too long path lengths at compile time', ex)
        #         else:
        #             raise
        # else:
        #     with open(path, "wb") as f:
        #         return f.write(data)
        with open(path, "wb") as f:
            return f.write(data)

    def customize_remove(self, path):
        path = self.__post_process_path(path)
        # old_cwd = os.getcwd()
        # try:
        #     os.chdir(os.path.dirname(path))
        #
        #     new_path = './' + path.split(self.customize_sep())[-1]
        #     try:
        #         os.remove(new_path)
        #     except WindowsError as ex:
        #         raise WindowsError('path {} error! {}'.format(path, ex))
        # finally:
        #     os.chdir(old_cwd)
        # if len(path) > 256:
        #     path = '\\\\?\\' + path
        os.remove(path)

    def customize_make_dir(self, path):
        path = self.__post_process_path(path)
        os.makedirs(path)

    def customize_remove_dir(self, path):
        path = self.__post_process_path(path)
        os.rmdir(path)

    def customize_sep(self):
        return os.sep

    def customize_is_dir(self, path):
        path = self.__post_process_path(path)
        return os.path.isdir(path)

    def customize_is_file(self, path):
        path = self.__post_process_path(path)
        return os.path.isfile(path)

    def customize_walk(self):
        for root, dirs, files in os.walk(self.root):
            root = root.split(self.customize_sep())[len(self.root.split(self.customize_sep())):]
            root = self.customize_sep().join(root)
            yield root, dirs, files


class DirectoryRes(BaseRes):
    def __init__(self, res, dir_):
        """

        :type res: BaseRes
        :type dir_: AnyStr
        """
        self.res = res
        self.dir = self.to_std_sep(dir_)
        self.dir_path_len = len(self.dir.split('/'))

    def __str__(self):
        return str(self.__unicode__())

    def __unicode__(self):
        return "{}(res={}, dir={})".format(self.__class__.__name__, self.res, repr(self.dir))

    def new_customize_path(self, p):
        return self.customize_sep().join((self.dir, p))

    def new_std_path(self, p):
        return '/'.join((self.dir, p))

    def customize_read(self, path):
        return self.res.read(self.new_customize_path(path))

    def customize_write(self, path, data):
        return self.res.write(self.new_customize_path(path), data)

    def customize_remove(self, path):
        return self.res.customize_remove(self.new_customize_path(path))

    def customize_make_dir(self, path):
        return self.res.customize_make_dir(self.new_customize_path(path))

    def customize_remove_dir(self, path):
        return self.res.customize_remove_dir(self.new_customize_path(path))

    def customize_sep(self):
        return self.res.customize_sep()

    def customize_is_dir(self, path):
        return self.res.customize_is_dir(self.new_customize_path(path))

    def customize_is_file(self, path):
        return self.res.customize_is_file(self.new_customize_path(path))

    def customize_walk(self):
        return [
            ('/'.join(root.split('/')[self.dir_path_len:]), dirs, files)
            for root, dirs, files in self.res.walk(self.dir)
        ]


class MergeRes(BaseRes):
    def __init__(self, *res):
        """

        :type res: BaseRes
        """
        self.res = res

    def customize_read(self, path):
        for r in self.res:
            try:
                return r.read(path)
            except:
                pass
        raise ResException("找不到对应文件")

    def customize_write(self, path, data):
        raise RuntimeError('写入相关方法不可用')

    def customize_remove(self, path):
        raise RuntimeError('写入相关方法不可用')

    def customize_make_dir(self, path):
        raise RuntimeError('写入相关方法不可用')

    def customize_remove_dir(self, path):
        raise RuntimeError('写入相关方法不可用')

    def customize_sep(self):
        return '/'

    def customize_is_dir(self, path):
        return any((i.is_dir(i.to_customize_sep(path)) for i in self.res))

    def customize_is_file(self, path):
        return any((i.is_file(i.to_customize_sep(path)) for i in self.res))

    def customize_walk(self):
        return [t for i in self.res for t in i.walk()]



# -*-coding:utf-8 -*-
u"""
:创建时间: 2021/12/24 15:12
:作者: 苍之幻灵
:我的主页: https://cpcgskill.com
:QQ: 2921251087
:爱发电: https://afdian.net/@Phantom_of_the_Cang
:aboutcg: https://www.aboutcg.org/teacher/54335
:bilibili: https://space.bilibili.com/351598127

"""
from __future__ import unicode_literals, print_function

from pyeal.res import BaseRes
from pyeal.exc import *


def file_ext(filename, sep='/'):
    return "." + ".".join(filename.split(sep)[-1].split(".")[1:])


def file_name(filename):
    return filename.split(".")[0]


class ModuleData(object):
    def __init__(self, source):
        """
        :type source: BaseRes
        """
        self.source = source
        self.__check()
        self.__module_index_table = self.__init_module_index_table()
        self.__lib_names = self.__init_lib_names()

    def __check_if_folder_is_package(self, path):
        if path == '':
            return False
        path = path.split('/')
        for i in range(1, len(path) + 1):
            loc_path = '/'.join(path[:i])
            loc_path = '/'.join([loc_path, '__init__.py'])
            if not self.source.is_file(loc_path):
                return False
        return True

    def __check(self):
        module_names = set()
        for root, dirs, files in self.source.walk():
            if self.__check_if_folder_is_package(root):
                module_name = ".".join(root.split('/'))
                if module_name in module_names:
                    raise ModuleDataException("存在多个解析名称为{}的文件或文件夹".format(module_name))
                module_names.add(module_name)
            elif root == '':
                for f in files:
                    if file_ext(f, '/') == ".py":
                        module_name = file_name(f)
                        if module_name in module_names:
                            raise ModuleDataException("存在多个解析名称为{}的文件或文件夹".format(module_name))
                        module_names.add(module_name)

    def __init_module_index_table(self):
        table = dict()
        for root, dirs, files in self.source.walk():
            # 检查包
            if self.__check_if_folder_is_package(root):
                table[".".join(root.split('/'))] = '/'.join((root, "__init__.py"))
                # 检查模块
                for f in files:
                    if file_ext(f, '/') == ".py":
                        f_name = file_name(f)
                        f_name = ".".join('/'.join((root, f_name)).split('/'))
                        table[f_name] = '/'.join((root, f))
            elif root == '':
                # 检查模块
                for f in files:
                    if file_ext(f, '/') == ".py":
                        f_name = file_name(f)
                        table[f_name] = f
        return table

    def __init_lib_names(self):
        return set((i.split(".")[0] for i in self.__module_index_table.keys()))

    def lib_names(self):
        return (i for i in self.__lib_names)

    def module_names(self):
        return (i for i in self.__module_index_table.keys())

    def module_name_and_paths(self):
        return ((k, v) for k, v in self.__module_index_table.items())

    def find_lib(self, n):
        if n in self.__lib_names:
            return n
        return None

    def find_module(self, n):
        if n in self.__module_index_table:
            return n
        return None

    def relative_find_module(self, n, parent_module):
        parent_names = parent_module.split(".")
        for m in self.__module_index_table.keys():
            if len(m.split(".")) > len(parent_names):
                if parent_module == ".".join(m.split(".")[:len(parent_names)]):
                    if n == ".".join(m.split(".")[len(parent_names):]):
                        return m
        return None

    def get_module_path(self, n):
        return self.__module_index_table[n]

    def __str__(self):
        body = "\n    ".join(("{}: {}".format(k, v) for k, v in self.__module_index_table.items()))
        return self.__class__.__name__ + "{\n    %s\n}" % body

    def __repr__(self):
        return repr(self.__str__())
